/*
 * Created on Jan 24, 2004
 *
 * @author Junaed Sattar
 * 
 */
package org.junaed.comp644;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Ellipse2D;
import javax.swing.JPanel;
import java.util.*;

/**
 * Custom Panel for curve drawing and quantization   
 * Grid drawing and resize support
 */
public class CurvePanel
	extends JPanel
	implements MouseListener, MouseMotionListener
{	
	private boolean isClear=false, isDynamic=false;
	private int nResolution, nType;
	private Vector pv,mv;
	private boolean showCurve=true; 
	private boolean showGrid=false, showCells=false;
	
	//testing alternate drawing method	
		

	/**
	 * CurvePanel Default Constructor
	 */
	public CurvePanel() {
		super();
		initialize();
		pv = new Vector();
		mv = new Vector();
		addMouseListener(this);
		addMouseMotionListener(this);		
	}
	
	/*
	 * public void drawCells(Graphics2D g2d)
	 * Draws the Freeman cells
	 * 
	 */
	public void drawCells(Graphics2D g2d)
	{		
		int maxX=this.getWidth();
		int maxY=this.getHeight();
		int nh = nResolution/2;
		
		g2d.setColor(Color.darkGray);
		switch(nType)
		{
//		Square quantization scheme visualization
		case 0:
		for( int i=nResolution-nh; i<maxY; i+=nResolution )
		{
			g2d.drawLine(1,i,maxX,i);
		}
		for( int i=nResolution-nh; i<maxX; i+=nResolution )
		{
			g2d.drawLine(i,0,i,maxY);
		}		
		break;
//		Circular quantization scheme visualization		
		case 1:
		for( int i=-nh; i<maxY; i+=nResolution )
		{
			for( int j=-nh; j<maxX; j+=nResolution )
			{				
				g2d.drawOval(j,i,nResolution,nResolution);
			}
		}
		break;
//		Rhombic/Grid-Intersect quantization scheme visualization 
		case 2:
		for( int i=-nh; i<maxY; i+=nResolution )
		{
			for( int j=-nh; j<maxX; j+=nResolution )
			{				
				int [] xpoints = {j, j+nh, j, j-nh, j};
				int [] ypoints = {i-nh, i, i+nh, i, i-nh};
				Polygon pg = new Polygon( xpoints, ypoints, 5 );
				g2d.draw(pg);				
			}
		}
		break;	
		}		
	}
	
	/*
	 * public void drawGrid(Graphics2D g2d)
	 * 
	 * Draws a 2D grid in gray given a resolution
	 */
	public void drawGrid(Graphics2D g2d)
	{		
		int maxX=this.getWidth();
		int maxY=this.getHeight();
		
		g2d.setColor(Color.gray);
		for( int i=nResolution; i<maxY; i+=nResolution )
		{
			g2d.drawLine(1,i,maxX,i);
		}
		for( int i=nResolution; i<maxX; i+=nResolution )
		{
			g2d.drawLine(i,0,i,maxY);
		}		
	}

	/*
	 * Core function for detecting mesh points
	 * to approximate the line drawing
	 * Uses square and circular quantization methods 
	 * to detect proximity to points on the line and 
	 * thereby choose the closest mesh point to approximate
	 * the continous line. 
	 */
	
	public void getMeshPoints()
	{
		int maxX=this.getWidth();
		int maxY=this.getHeight();
		int nh = nResolution/2;
				
		if( pv.isEmpty()) 
		{
			javax.swing.JOptionPane.showMessageDialog( null, "Draw something first!");
			return;
		}
		mv.clear();
		for( int i=nResolution; i<maxY; i+=nResolution )
		{
			for( int j=nResolution; j<maxX; j+=nResolution )
			{				 
				
				switch (nType)
				{
					case 0:
						Rectangle r =  new Rectangle(j-nh, i-nh, nResolution, nResolution );
						for( int k=0; k<pv.size(); k++ )
						{
							Point p = (Point)pv.elementAt(k);
							if( p.x==1000 ) continue;
//							check for duplicates, maybe?							
							if( r.contains(p)) mv.add(new Point(j,i));
						}
						break;
					case 1:
						Ellipse2D.Double c =  new Ellipse2D.Double(j-nh, i-nh, nResolution, nResolution);
						for( int k=0; k<pv.size(); k++ )
						{
							Point p = (Point)pv.elementAt(k);
//							check for duplicates, maybe?
							if( c.contains(p.x, p.y)) mv.add(new Point(j,i));
						}
						break;
					case 2:
						int [] xpoints = {j, j+nh, j, j-nh, j};
						int [] ypoints = {i-nh, i, i+nh, i, i-nh};
						Polygon pg = new Polygon( xpoints, ypoints, 5 );
						for( int k=0; k<pv.size(); k++ )
						{
							Point p = (Point)pv.elementAt(k);
							if( p.x==1000 ) continue;
//							check for duplicates, maybe?							
							if( pg.contains(p)) mv.add(new Point(j,i));
						}
						
						break;										
				}
			}		
		}
	}
	
	/**
	 * @return the current grid resolution 
	 */
	public int getNResolution() {
		return nResolution;
	}

	/**
	 * @return the current quantization type 
	 */
	public int getNType() {
		return nType;
	}

	/**
	 * This method initializes the visual interface
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "", javax.swing.border.TitledBorder.LEFT, javax.swing.border.TitledBorder.TOP, new java.awt.Font("Dialog", java.awt.Font.BOLD, 12), java.awt.Color.cyan));
		this.setSize(300, 200);
		this.setBackground(java.awt.Color.black);
		/* My code */
		nResolution=20;
	}
	/**
	 * @return
	 */
	public boolean IsDynamic() {
		return isDynamic;
	}
	/**
	 * @return true if cell display is on, false otherwise
	 */
	public boolean isShowCells() {
		return showCells;
	}

	/**
	 * @return
	 */
	public boolean isShowCurve() {
		return showCurve;
	}

	/**
	 * @return true if grid display is on, false otherwise
	 */
	public boolean isShowGrid() {
		return showGrid;
	}

	public void mouseClicked(MouseEvent arg0) 
	{
/*		Point p = arg0.getPoint();
		pv.add(p);
		repaint();		
*/	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent arg0) 
	{
		Point p = arg0.getPoint();
		pv.add(p);
		repaint();
	}

	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	public void mouseMoved(MouseEvent arg0) {}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) 
	{
		Point p = arg0.getPoint();
		pv.add(p);
		repaint();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) 
	{
		Point p = new Point(1000,1000);
		pv.add(p);
		System.out.println("Point vector is now at "+pv.size());
		repaint();
	}

	/* Actual method for painting the curves and everything
	 * Heart of the Class
	 * @author Junaed Sattar
	 * @param g: the graphics (Java 1) context
	 */
	protected void paintComponent(Graphics g) 
	{
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D)g.create();
//		Draw Background 
		if (isOpaque()) 
		{
			g2d.setColor(Color.black);
			g2d.fillRect(0, 0, getWidth(), getHeight());
		}
//		Draw Grid and or cells  
		if( showGrid ) drawGrid(g2d);
		if( showCells ) drawCells(g2d);
//		See if clear button has been pressed
		if( isClear )
		{
			isClear=false;
			// Reset clear flag, empty point and chain vectors 
			pv.clear();
			mv.clear();
			// Dispose of graphics2D object, and return
			g2d.dispose();
			return;
		}
//		While there are points to draw...
		if( !pv.isEmpty() && showCurve )
		{
//			if dynamic update is on, immidiately update the mesh 
			if ( isDynamic ) getMeshPoints();
//			Draw Lines between successive points in the vector
			g2d.setPaint(Color.white);
			for( int i=0; i<pv.size()-1; i++ )
			{
				Point p = (Point)pv.elementAt(i);
				Point q = (Point)pv.elementAt(i+1);
//				Break only when the user has lifted the mouse off the canvas
				if( q.x==1000 || p.x==1000 ) continue;
				g2d.drawLine(p.x, p.y, q.x, q.y);		
			}
		}
//		if there are points in the Freeman vector, display them
		if( !mv.isEmpty())
		{
			g2d.setPaint(Color.magenta);
			for( int i=0; i<mv.size()-1; i++ )
			{
				Point p = (Point)mv.elementAt(i);
				g2d.fillRect(p.x-2, p.y-2, 4, 4);		
			}
			
		}
//		dispose the temporary graphics object
		g2d.dispose();
	}
	
	/**
	 * @param b: toggles clear flag on or off
	 */
	public void setClear(boolean b) {
		isClear = b;
	}

	/**
	 * @param b:toggles the dynamic update flag on or off
	 */
	public void setDynamic(boolean b) {
		isDynamic = b;
	}

	/**
	 * @param i: sets the current grid resolution
	 */
	public void setNResolution(int i) {
		nResolution = i;
	}

	/**
	 * @param i: sets the current quantization type
	 */
	public void setNType(int i) {
		nType = i;
	}

	/**
	 * @param b:toggles the cell show flag on or off
	 */
	public void setShowCells(boolean b) {
		showCells = b;
	}

	/**
	 * @param b
	 */
	public void setShowCurve(boolean b) {
		showCurve = b;
	}

	/**
	 * @param b:toggles the grid show flag on or off
	 */
	public void setShowGrid(boolean b) {
		showGrid = b;
	}
}
